#include <stdio.h>
#include <string.h>

#define HLOUPE

int l, w, h;
char posl[512];
char mapa[512][512];	/* [y][x] */
int kroku[512][512][4]; /* [y][x][d] */
/* -1 neznamy, 0 zacykleni, -2 prave pocitam, >0 pocet kroku */

#define MAPA(y,x) ( ((x>=0)&&(x<w)&&(y>=0)&&(y<h)) ? mapa[(y)][(x)] : 'G' )

/* prove jeden krok mapou */
void inline krok(int *y, int *x, int *dir, int *mod)
{
  char c = posl[*mod];
  int nx = *x, ny = *y;
  const int xd[] = {0,1,0,-1};
  const int yd[] = {-1,0,1,0};
  *mod = ((*mod)+1) % l;
  switch (c)
  { 
    case 'R': *dir = ((*dir)+1) % 4; return;
    case 'L': *dir = ((*dir)+3) % 4; return;
    case '+': nx += xd[*dir]; ny += yd[*dir]; break;
    case '-': nx -= xd[*dir]; ny -= yd[*dir]; break;
  }
  if (MAPA(ny,nx) != '#') {*x = nx; *y = ny; }
}

/* i velmi pomal een O(w^2 * h^2 * l) je nkdy dost rychl */
void hloupe(int y, int x)
{
  int k = 0;
  /* Rychlej robot */
  int ax = x, ay = y, adir = 0, amod = 0; 
  /* 2x pomalej robot */
  int bx = x, by = y, bdir = 0, bmod = 0; 
  kroku[y][x][0] = 0;
  if (MAPA(y, x)=='#') 
  {
    kroku[y][x][0] = 0;
    return;
  }
  while (MAPA(ay, ax)!='G')
  { 
    krok(&ay, &ax, &adir, &amod);
    if((k%2)==1) /* 2x pomalej */
      krok(&by, &bx, &bdir, &bmod);
    if ((ax==bx)&&(ay==by)&&(adir==bdir)&&(amod==bmod)) 
    { /* pokud se potkali (poloha, smr, poloha v sekvenci), jsou v cyklu */
      kroku[y][x][0] = 0;
      return;
    }
    k++;
  }
  kroku[y][x][0] = k;
}

/* Prove (nanejv) l krok, 
 * pokud dojde do cle, vra poet krok, jinak 0 */
int inline l_kroku(int *y, int *x, int *d)
{
  int s, mod = 0;
  for (s=1; s<=l; s++)
  {
    krok(y, x, d, &mod);
    if (MAPA(*y, *x) == 'G') 
      return s;
  }
  return 0;
}

/* Chytr een v O(w * h * l) */

/* zasobnik pozic -- moc velky, aby se vesel na programovy zasobnik */
#define STACK (512*512*4)
int xstack[STACK];
int ystack[STACK];
int dstack[STACK];
int stack;

void chytre(int y, int x)
{
  int d = 0, s = 0;
  if (MAPA(y, x)=='#') 
  {
    kroku[y][x][0] = 0;
    return;
  }
  stack = 0;
  /* dokud je stav neznm */
  while(s==0 && kroku[y][x][d]==-1)
  {
    kroku[y][x][d] = -2;
    /* ulo tuto pozici */
    xstack[stack] = x;
    ystack[stack] = y;
    dstack[stack] = d;
    stack++;
    /* prove (nanejv) l pkaz */
    s = l_kroku(&y, &x, &d);
  }
  /* doli jsme na znmou hodnotu? */
  if (s==0 && kroku[y][x][d]>0)
    s = kroku[y][x][d] + l;
  /* s == 0 prave tehdy, kdy jsme doli do smyky */
  for (stack--; stack>=0; stack--)
  {
    /* nastav sprvnou vzdlenost vem zapamatovanm mstm */
    kroku[ystack[stack]][xstack[stack]][dstack[stack]] = s;
    if (s>0) s += l; /* jen pokud nejsme ve smyce... */
  }
}

int main(void)
{
  int i, j, d, uspechu = 0;
  FILE *f;

  /* vstup */
  f = fopen("akce.in", "r");
  fscanf(f, "%d", &l);
  fscanf(f, "%s", posl);
  fscanf(f, "%d %d", &w, &h);
  for (i=0; i<h; i++)
    fscanf(f, "%s", mapa[i]);
  fclose(f);

  /* inicializace kroku[][][] */
  for (i=0; i<h; i++)
    for (j=0; j<w; j++)
      for (d=0; d<4; d++)
	kroku[i][j][d] = -1;

  /* vpoet */
  for (i=0; i<h; i++)
    for (j=0; j<w; j++)
    {
#ifdef HLOUPE
      hloupe(i, j);
#else
      chytre(i, j);
#endif
      if (kroku[i][j][0]>0) uspechu++;
    }

  /* vstup */
  f = fopen("akce.out", "wt");
  fprintf(f, "%d\n", uspechu);
  for (i=0; i<h; i++)
  {
    fprintf(f, "%d", kroku[i][0][0]);
    for (j=1; j<w; j++)
      fprintf(f, " %d", kroku[i][j][0]);
    fprintf(f, "\n");
  }
  fclose(f);

  return 0;
}
